# -*- coding: utf-8 -*-
# @微信:samequant
# @网站:打板哥网 www.dabange.com 
# @更多源码下载地址: https://dabange.com/download
# @有偿服务：量化入门培训、量化数据爬取、策略代写
# @日期: 2024/3/24
"""
miniqmt业内俗称小QMT，应该是最适合散户量化交易的方式，没有之一！相比于大qmt，他可以直接在python编辑器如pycharm里写策略，同时支持免费查实时和历史行情数据，支持查资金成交持仓委托，以及买卖下单和撤单等。几乎可以满足量化交易的全部基本需求。今天分享，如果快速上手miniqmt，并演示买卖下单！

miniqmt使用步骤：
第一步，你需要联系你所在的券商，申请开通含有极简模式即miniqmt的QMT终端权限。
第二步，本机安装好qmt，记录好安装路径，然后在site-packages下找到xtquant文件夹，复制，然后粘贴到你本机python的site-packages文件夹下。
第三部，在python代码编辑器导入所需的包。这样，就可以直接在python里调用miniqmt的行情和交易接口啦！
"""
import pandas as pd
pd.set_option('expand_frame_repr', False)  # 当列太多时显示完整
pd.set_option('display.max_rows', 5000)  # 最多显示数据的行数

import os
import time
from qmt_config import *
from xtquant.xttype import StockAccount
from xtquant.xttrader import XtQuantTrader, XtQuantTraderCallback
from xtquant import xtdata
from xtquant import xtconstant



class Miniqmt_stock(object):
    def __init__(self, account_id=None, account_type='STOCK'):
        # 创建股票账户
        self.account = StockAccount(account_id, account_type)
        # 获取该class类所在.py文件的绝对目录
        self.file_full_dir = os.path.dirname(os.path.abspath(__file__))

        session_id = int(time.time() * 1000)  # session_id为会话编号，策略使用方对于不同的Python策略需要使用不同的会话编号（自己随便写）
        self.xt_trader = XtQuantTrader(path, session_id)  # 创建API实例
        # 启动交易线程
        self.xt_trader.start()
        # 建立交易连接，返回0表示连接成功
        connect_result = self.xt_trader.connect()
        if connect_result == 0:
            print('交易连接成功！')
        # 对交易回调进行订阅，订阅后可以收到交易主推，返回0表示订阅成功
        subscribe_result = self.xt_trader.subscribe(self.account)
        # print(subscribe_result)

    def query_asset(self):
        # 查询账户资产资金
        asset = self.xt_trader.query_stock_asset(self.account)
        asset_dict = {'总资产': asset.total_asset, '可用资金': asset.cash, '冻结金额': asset.frozen_cash,
                      '持仓市值': asset.market_value}
        return asset_dict

    def quert_positions(self):
        positions = self.xt_trader.query_stock_positions(self.account)

        # 查询账户持仓明细
        def position_to_dict(i):
            return {
                '证券代码': i.stock_code,
                '持仓数量': i.volume,
                '可用数量': i.can_use_volume,
                '平均建仓成本': i.open_price,
                '市值': i.market_value
            }

        if len(positions) > 0:
            df_pos = pd.DataFrame(list(map(position_to_dict, positions)))
            return df_pos
        else:
            print('当前无持仓信息')
            return pd.DataFrame()

    def query_orders(self):
        # 查账户的委托明细
        orders = self.xt_trader.query_stock_orders(self.account, False)

        def order_to_dict(i):
            return {
                '证券代码': i.stock_code,
                '委托时间': i.order_time,
                '委托数量': i.order_volume,
                '成交数量': i.traded_volume,
                '委托价格': i.price,
                '订单编号': i.order_id,
                '委托状态': i.order_status,
                '委托类型': i.order_type
            }

        if len(orders) != 0:
            orders_lst = list(map(order_to_dict, orders))
            df_orders = pd.DataFrame(orders_lst)
            return df_orders
        else:
            print('当前无委托信息')
            return pd.DataFrame()

    def query_deals(self):
        # 查询当日最近的成交
        deals = self.xt_trader.query_stock_trades(self.account)

        def trade_to_dict(i):
            return {
                '证券代码': i.stock_code,
                '委托类型': i.order_type,
                '成交时间': i.traded_time,
                '成交均价': i.traded_price,
                '成交数量': i.traded_volume,
                '成交金额': i.traded_amount,
                '订单编号': i.order_id,
                '策略名称': i.strategy_name,
            }

        if len(deals) != 0:
            deals_lst = list(map(trade_to_dict, deals))
            df_trades = pd.DataFrame(deals_lst)
            return df_trades
        else:
            print('当前无成交信息')
            return pd.DataFrame()

    def get_quote(self, stock_lst=[]):
        # 获取股票全推实时行情
        quote_dict = xtdata.get_full_tick(code_list=stock_lst)
        # print(quote_dict)
        if quote_dict:
            df = pd.DataFrame(quote_dict)
            df = df.T
            df.reset_index(inplace=True)
            rename_dict = {'index': '证券代码', 'timetag': '时间', 'lastPrice': '最新价', 'open': '开盘价', 'high': '最高价',
                           'low': '最低价',
                           'lastClose': '昨收价',
                           'amount': '成交额', 'volume': '成交量', 'pvolume': '原始成交总量',
                           'settlementPrice': '今结算', 'lastSettlementPrice': '前结算', 'askPrice': '五档卖价',
                           'bidPrice': '五档买价',
                           'askVol': '五档卖量', 'bidVol': '五档买量'}
            df.rename(columns=rename_dict, inplace=True)
            # print(df.columns)
            df = df[['证券代码', '时间', '最新价', '开盘价', '最高价', '最低价', '昨收价', '成交额', '成交量', '五档卖价', '五档买价', '五档卖量', '五档买量']]
            return df
        else:
            return pd.DataFrame()

    def order_buy(self, stock_code=None, buy_money=None, price_type='最新价'):
        if stock_code == None:
            print('代码不能为空！')
            return
        # 按最新价买入下单，接口返回订单编号，后续可以用于撤单操作以及查询委托状态
        df_quote = self.get_quote(stock_lst=[stock_code])

        new_price = df_quote.loc[0, '最新价']
        sell_1_price = df_quote.loc[0, '五档卖价'][0]
        sell_2_price = df_quote.loc[0, '五档卖价'][1]
        sell_3_price = df_quote.loc[0, '五档卖价'][2]
        sell_4_price = df_quote.loc[0, '五档卖价'][3]
        sell_5_price = df_quote.loc[0, '五档卖价'][4]

        if price_type == '最新价':
            price = new_price
        elif price_type == '卖一价':
            price = sell_1_price
            if sell_1_price == 0:
                price = new_price
        elif price_type == '卖二价':
            price = sell_2_price
            if sell_2_price == 0:
                price = new_price
        elif price_type == '卖三价':
            price = sell_3_price
            if sell_3_price == 0:
                price = new_price
        elif price_type == '卖四价':
            price = sell_4_price
            if sell_4_price == 0:
                price = new_price
        elif price_type == '卖五价':
            price = sell_5_price
            if sell_5_price == 0:
                price = new_price
        else:
            print('买入委托价格类型错误，请检查！')
            return
        buy_num = int(buy_money / (price * 100)) * 100
        fix_result_order_id = self.xt_trader.order_stock(self.account, stock_code, xtconstant.STOCK_BUY, buy_num,
                                                         xtconstant.LATEST_PRICE, -1, '通达信自动交易', '买入')
        if fix_result_order_id:
            print(f'买入{stock_code}数量{buy_num}股，委托编号{fix_result_order_id}')

    def order_sell(self, stock_code=None, sell_num=None, price_type='买一价'):
        if stock_code == None:
            print('代码不能为空！')
            return
        if sell_num == None:
            print('卖出数量不能为空！')
            return
        # 按最新价买入下单，接口返回订单编号，后续可以用于撤单操作以及查询委托状态
        df_quote = self.get_quote(stock_lst=[stock_code])

        new_price = df_quote.loc[0, '最新价']
        buy_1_price = df_quote.loc[0, '五档买价'][0]
        buy_2_price = df_quote.loc[0, '五档买价'][1]
        buy_3_price = df_quote.loc[0, '五档买价'][2]
        buy_4_price = df_quote.loc[0, '五档买价'][3]
        buy_5_price = df_quote.loc[0, '五档买价'][4]

        if price_type == '最新价':
            price = new_price
        elif price_type == '买一价':
            price = buy_1_price
            if buy_1_price == 0:
                price = new_price
        elif price_type == '买二价':
            price = buy_2_price
            if buy_2_price == 0:
                price = new_price
        elif price_type == '买三价':
            price = buy_3_price
            if buy_3_price == 0:
                price = new_price
        elif price_type == '买四价':
            price = buy_4_price
            if buy_4_price == 0:
                price = new_price
        elif price_type == '买五价':
            price = buy_5_price
            if buy_5_price == 0:
                price = new_price
        else:
            print('卖出委托价格类型错误，请检查！')
            return

        fix_result_order_id = self.xt_trader.order_stock(self.account, stock_code, xtconstant.STOCK_SELL, sell_num,
                                                         price, -1, '通达信自动交易', '卖出')
        if fix_result_order_id:
            print(f'卖出{stock_code}数量{sell_num}股，委托编号{fix_result_order_id}')

    def download_history_data(self, stock_code=None, period='1d', start_time=None, end_time=None):
        xtdata.download_history_data(stock_code=stock_code, period=period, start_time=start_time,
                                     end_time=end_time)


if __name__ == '__main__':
    m_qmt_1 = Miniqmt_stock(account_id=putong_account_1)

    # 查询账户资产资金
    asset_dict = m_qmt_1.query_asset()
    print(asset_dict)
    # 多账户委托
    # m_qmt_2 = Miniqmt_stock(account_id=putong_account_2)
    # asset_dict2 = m_qmt_2.query_asset()
    # print(asset_dict2)
    # exit()

    # 查询账户持仓明细
    df_pos = m_qmt_1.quert_positions()
    print(df_pos)

    # 查账户的委托明细
    # df_orders = m_qmt_1.query_orders()
    # print(df_orders)

    # 查账户的成交明细
    # df_deals = m_qmt_1.query_deals()
    # print(df_deals)

    # 查实时全推行情
    # df_quote = m_qmt_1.get_quote(stock_lst=['000001.SZ', '600519.SH'])
    # print(df_quote)

    # 下载历史行情到本地 极简版QMT的路径下的datadir文件夹内 D:\兴业证券SMT-Q\userdata_mini\datadir
    # m_qmt_1.download_history_data(stock_code='600519.SH', period='1d', start_time='20240201', end_time='20240323')

    # 读取本地历史行情数据
    # code_list = ['000001.SZ', '600519.SH']
    # history_data = xtdata.get_market_data_ex([], code_list, period='1d', count=-1)
    # print(history_data)

    # 查实时全推行情 传入市场代码代表订阅全市场，示例：['SH', 'SZ'] 或 传入合约代码代表订阅指定的合约，示例：['600000.SH', '000001.SZ']
    # code_list = ['SH', 'SZ']
    # full_tick = xtdata.get_full_tick(code_list=code_list)
    # print('全推数据日线最新值', full_tick)
